#!/bin/bash
# Copyright 2008-2010 Matus Chochlik. Distributed under the Boost
# Software License, Version 1.0. (See accompanying file
# LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
#
# read filepaths from standard input and prints only
# those which actually exist in the directories passed as arguments
function print_existing_in()
{
	while read input
	do
		for file in ${input}
		do
			for dir in $*
			do
				if [ -f ${dir}/${file} ]
				then echo -e "\t${dir}/${file} \\"
				fi
			done
		done
	done
}
#
# a dummy file that all other source/header files "depend" on
# it can be used to force rebuild even if no real source changes
force_target="out/.force_rebuild"
#
# print the copyright and license notice to the output
source $(dirname $0)/_make_disclaimer.sh
#
# go through input directories
for dir
do
	# find all .cpp/.hpp files and read their paths
	find ${dir} -type f -name '*.[ch]pp' -o -name '*.inc' |
	sort |
	while read filepath
	do
		# create a Makefile rule for the current source file
		#echo ".PHONY: ${filepath}"
		#echo
		echo "${filepath}: \\"
		# list all files included from the source file that can
		# be found in one of the input directories as dependencies
		# for the target file:
		# for #include <...>
		sed -n 's/^#\s*include\s*<\([^>]\+\)>/\1/p' < ${filepath} |
		print_existing_in $* .
		# and for #include "..."
		sed -n 's/^#\s*include\s*"\([^"]\+\)"/\1/p' < ${filepath} |
		sed 's:/\./:/:g;s:^\./::g' |
		print_existing_in $(dirname ${filepath})
		# add the rebuild forcing target as dependency
		echo -e "\t${force_target}"
		# finish the rule
		echo -e "\t@touch ${filepath}"
		echo
	done
done
#
echo "${force_target}: out/.dir_dummy"
echo -e "\t@touch ${force_target}"
echo

#
echo ".PHONY: force_bin_rebuild"
echo
echo "force_bin_rebuild:"
echo -e "\trm -f ${force_target}"
