<!--
 Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 Software License, Version 1.0. (See accompanying file
 LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
-->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exsl="http://exslt.org/common"
  extension-element-prefixes="exsl"
  version="1.0"
>
  <xsl:output method="text"/>
  <xsl:strip-space elements="*"/>
  <xsl:preserve-space elements="xsl:text"/>

  <xsl:template name="newline">
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="*">
  </xsl:template>

  <xsl:template name="get_ancestor_count">
    <xsl:param name="id"/>
    <xsl:variable name="temp">
      <v>0</v>
      <xsl:for-each select="/doxygen/compounddef[@id = $id]/basecompoundref">
        <v>1</v>
        <v>
          <xsl:call-template name="get_ancestor_count">
            <xsl:with-param name="id" select="@refid"/>
          </xsl:call-template>
        </v>
      </xsl:for-each>
    </xsl:variable>
    <xsl:value-of select="sum(exsl:node-set($temp)/*)"/>
  </xsl:template>

  <xsl:template match="/doxygen">
    <xsl:text>
      digraph "Mirror Concepts" {
      rankdir="LR";
      ratio="compress";
      mode="maxent";
      node [fontname="Courier"];

      subgraph cluster_legend {
        id="legend";
        label="Legend";
        graph [truecolor bgcolor="#E0E0E8"];
        fontsize=48;
        "legend-metaobject1" [label="meta-object" shape="egg" fontsize="16" fontcolor="white" style="filled" fillcolor="#424246"];
        "legend-metaobject2" [label="meta-object" shape="egg" fontsize="16" fontcolor="white" style="filled" fillcolor="#424246"];
        "legend-metaobject1" -> "legend-metaobject2" [dir="both" arrowhead="none" arrowtail="normal" arrowsize=2.0 label="inheritance"];

        "legend-type" [label="type" shape="invtrapezium" style="filled" fillcolor="#f0f0f0"];
        subgraph "cluster_legend-typedef" {
          style="invis";
          "legend-typedef" [label="member typedef" shape="rectangle" style="filled" fillcolor="#CFDFFF"];
          "legend-typedef-hook" [shape="point"];
          "legend-typedef-hook" -> "legend-typedef" [constraint=false arrowhead="none" style="dotted" len="0.2"];
        }
        subgraph "cluster_legend-function" {
          style="invis";
          "legend-function" [label="member function" shape="ellipse" style="filled" fillcolor="#B0C0E0"];
          "legend-function-hook" [shape="point"];
          "legend-function-hook" -> "legend-function" [constraint=false arrowhead="none" style="dotted"];
        }
        subgraph "cluster_legend-metafn" {
          style="invis";
          "legend-metafn" [label="metafunction template class" shape="hexagon" style="filled" fillcolor="#B8C8E8"];
          "legend-metafn-hook" [shape="point"];
          "legend-metafn-hook" -> "legend-metafn" [constraint=false arrowhead="none" style="dotted"];
        }

        "legend-metaobject1" -> "legend-typedef-hook" [arrowhead="none" arrowsize=2.0];
        "legend-typedef-hook" -> "legend-metaobject2" [arrowhead="vee" arrowsize=2.0];

        "legend-metaobject1" -> "legend-metafn-hook" [dir="both" arrowhead="none" arrowtail="diamond" arrowsize=2.0];
        "legend-metafn-hook" -> "legend-metaobject2" [arrowhead="vee" arrowsize=2.0];

        "legend-metaobject1" -> "legend-function-hook" [arrowhead="none" arrowsize=2.0];
        "legend-function-hook" -> "legend-type" [arrowhead="vee" arrowsize=2.0];
      }
      root="MetaObject";
    </xsl:text>

    <xsl:text>
      subgraph cluster_pos_hooks {
        style="invis";
    </xsl:text>
    <xsl:for-each select="compounddef[@kind='struct']">
      <!-- make the number node -->
      <xsl:text>"</xsl:text>
      <xsl:value-of select="position()"/>
      <xsl:text>" [shape="none" label=""];</xsl:text>

    <xsl:if test="following-sibling::compounddef[@kind='struct']">
      <xsl:text>"</xsl:text>
      <xsl:value-of select="position()"/>
      <xsl:text>" -> "</xsl:text>
      <xsl:value-of select="position()+1"/>
      <xsl:text>" [style="invis" weight=10];</xsl:text>
    </xsl:if>


    </xsl:for-each>
    <xsl:text>
      }
    </xsl:text>

    <xsl:apply-templates match="compounddef[@kind='struct']"/>

    <xsl:text>
        overlap=false;
        splines=true;
      }
    </xsl:text>
  </xsl:template>

  <xsl:template match="compounddef[@kind='struct' and not(contains(compoundname/text(), '::'))]">
    <!-- The type of the concept (meta-object, range, tag, etc.) -->
    <xsl:variable name="type">
      <xsl:choose>
        <xsl:when test="compoundname/text() = 'Range'">range</xsl:when>
        <xsl:when test="substring(compoundname/text(), 1, 4) = 'Meta'">metaobject</xsl:when>
        <xsl:when test="substring(compoundname/text(), string-length(compoundname/text())-2) = 'Tag'">tag</xsl:when>
        <xsl:otherwise>other</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!-- The concept name -->
    <xsl:variable name="struct_name" select="compoundname/text()"/>

    <xsl:variable name="ancestor_count">
      <xsl:call-template name="get_ancestor_count">
        <xsl:with-param name="id" select="@id"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="metaobject_rank" select="$ancestor_count div 2"/>

    <!-- make the concept node -->
    <xsl:text>"</xsl:text>
    <xsl:value-of select="$struct_name"/>
    <xsl:text>" [label="</xsl:text>
    <xsl:value-of select="$struct_name"/>
    <xsl:text>" id="</xsl:text>
    <xsl:value-of select="$struct_name"/>
    <xsl:text>" shape="</xsl:text>
    <xsl:choose>
      <xsl:when test="$type='tag'">septagon</xsl:when>
      <xsl:when test="$type='metaobject'">egg</xsl:when>
      <xsl:otherwise>invtrapezium</xsl:otherwise>
    </xsl:choose>
    <xsl:text>" fontsize="</xsl:text>
    <xsl:choose>
      <xsl:when test="$type='metaobject'"><xsl:value-of select="16+count(sectiondef/memberdef)+count(basecompoundref)"/></xsl:when>
      <xsl:when test="$type='tag'"><xsl:value-of select="15+count(basecompoundref)"/></xsl:when>
      <xsl:otherwise>14</xsl:otherwise>
    </xsl:choose>
    <xsl:text>" style="filled" fillcolor="</xsl:text>
    <xsl:choose>
      <xsl:when test="$type='tag'">#A0A0A0</xsl:when>
      <xsl:when test="$type='metaobject'">
        <xsl:choose>
          <xsl:when test="$metaobject_rank &lt; 1">#404040</xsl:when>
          <xsl:when test="$metaobject_rank &lt; 2">#383844</xsl:when>
          <xsl:when test="$metaobject_rank &lt; 3">#363648</xsl:when>
          <xsl:when test="$metaobject_rank &lt; 4">#34344B</xsl:when>
          <xsl:when test="$metaobject_rank &lt; 5">#323250</xsl:when>
          <xsl:when test="$metaobject_rank &lt; 6">#303054</xsl:when>
          <xsl:when test="$metaobject_rank &lt; 7">#282858</xsl:when>
          <xsl:when test="$metaobject_rank &lt; 8">#26265B</xsl:when>
          <xsl:when test="$metaobject_rank &lt; 9">#242460</xsl:when>
          <xsl:otherwise>#023653</xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>#f0f0f0</xsl:otherwise>
    </xsl:choose>
    <xsl:text>" fontcolor="</xsl:text>
    <xsl:choose>
      <xsl:when test="$type='metaobject'">white</xsl:when>
      <xsl:otherwise>black</xsl:otherwise>
    </xsl:choose>
    <xsl:text>"];</xsl:text>
    <xsl:call-template name="newline"/>

    <!-- link to the positional hooks -->
    <xsl:text>"</xsl:text>
    <xsl:value-of select="position()"/>
    <xsl:text>" -> "</xsl:text>
    <xsl:value-of select="$struct_name"/>
    <xsl:text>" [style="invis" weight=15];</xsl:text>


    <xsl:if test="following-sibling::compounddef[@kind='struct']">
      <xsl:text>"</xsl:text>
      <xsl:value-of select="$struct_name"/>
      <xsl:text>" -> "</xsl:text>
      <xsl:value-of select="position()+1"/>
      <xsl:text>" [style="invis" weight=10];</xsl:text>
    </xsl:if>

    <xsl:text>"</xsl:text>
    <xsl:value-of select="$ancestor_count+1"/>
    <xsl:text>" -> "</xsl:text>
    <xsl:value-of select="$struct_name"/>
    <xsl:text>" [style="invis" weight=25];</xsl:text>

    <!-- Process the base classes -->
    <xsl:for-each select="basecompoundref">
      <xsl:variable name="base_id" select="@refid"/>
      <xsl:text>"</xsl:text>
      <xsl:value-of select="/doxygen/compounddef[@id=$base_id]/compoundname/text()"/>
      <xsl:text>" -> "</xsl:text>
      <xsl:value-of select="$struct_name"/>
      <xsl:text>" [id="base_of#</xsl:text>
      <xsl:value-of select="$struct_name"/>
      <xsl:text>#</xsl:text>
      <xsl:value-of select="/doxygen/compounddef[@id=$base_id]/compoundname/text()"/>
      <xsl:text>" penwidth=0.4 arrowsize=2.5 dir="both" arrowhead="none" arrowtail="normal" weight=3];</xsl:text>
      <xsl:call-template name="newline"/>
    </xsl:for-each>

    <xsl:text>subgraph cluster_</xsl:text>
    <xsl:value-of select="$struct_name"/>
    <xsl:text> {</xsl:text>
    <xsl:call-template name="newline"/>
    <xsl:text>style="invis";</xsl:text>
    <xsl:call-template name="newline"/>

    <!-- Process the members -->
    <xsl:for-each select="sectiondef/memberdef">
      <xsl:variable name="is_metafn" select="contains(name/text(), '&lt;')"/>
      <xsl:variable name="member_name">
        <xsl:choose>
          <xsl:when test="$is_metafn">
            <xsl:value-of select="substring-before(name/text(), '&lt;')"/>
          </xsl:when>
          <xsl:otherwise><xsl:value-of select="name/text()"/></xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:if test="
        not($member_name='categorize_meta_object') and
        not(starts-with($member_name, 'operator'))
      ">

        <!-- The resulting type -->
        <xsl:variable name="result_name">
          <xsl:choose>
            <xsl:when test="detaileddescription/para/simplesect[@kind='return']/para/ref/text()">
              <xsl:value-of select="detaileddescription/para/simplesect[@kind='return']/para/ref/text()"/>
            </xsl:when>
            <xsl:when test="detaileddescription/para/simplesect[@kind='return']/para/text()">
              <xsl:value-of select="detaileddescription/para/simplesect[@kind='return']/para/text()"/>
            </xsl:when>
            <xsl:when test="type/ref/text()">
              <xsl:value-of select="type/ref/text()"/>
            </xsl:when>
          </xsl:choose>
        </xsl:variable>

        <!-- Check of it is a Range -->
        <xsl:variable name="is_aggr" select="starts-with($result_name, 'Range&lt;')"/>

        <!-- The name of the concept we are going point to -->
        <xsl:variable name="referent_name">
          <xsl:choose>
            <xsl:when test="$is_aggr">
              <xsl:value-of select="substring-before(substring-after($result_name, '&lt;'),'&gt;')"/>
            </xsl:when>
            <xsl:otherwise><xsl:value-of select="$result_name"/></xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:variable name="edge_name">
          <xsl:value-of select="@kind"/>
          <xsl:text>_</xsl:text>
          <xsl:value-of select="$struct_name"/>
          <xsl:text>_</xsl:text>
          <xsl:value-of select="$member_name"/>
        </xsl:variable>

        <xsl:variable name="edgelabel_qname">
          <xsl:text>"edgelabel-</xsl:text>
          <xsl:value-of select="$edge_name"/>
          <xsl:text>"</xsl:text>
        </xsl:variable>

        <xsl:variable name="edgehook_qname">
          <xsl:text>"edgehook-</xsl:text>
          <xsl:value-of select="$edge_name"/>
          <xsl:text>"</xsl:text>
        </xsl:variable>

        <xsl:text>subgraph cluster_</xsl:text>
        <xsl:value-of select="$edge_name"/>
        <xsl:text>
          {
            style="invis";
        </xsl:text>

        <xsl:value-of select="$edgelabel_qname"/>
        <xsl:text> [shape="</xsl:text>
        <xsl:choose>
          <xsl:when test="@kind='typedef'">rectangle</xsl:when>
          <xsl:when test="@kind='friend'">hexagon</xsl:when>
          <xsl:when test="@kind='function'">ellipse</xsl:when>
          <xsl:otherwise>trapezium</xsl:otherwise>
        </xsl:choose>
        <xsl:text>" label="</xsl:text>
        <xsl:value-of select="$member_name"/>
        <xsl:text>" style="filled" fillcolor="</xsl:text>
          <xsl:choose>
            <xsl:when test="@kind='typedef'">#CFDFFF</xsl:when>
            <xsl:when test="@kind='friend'">#B8C8E8</xsl:when>
            <xsl:when test="@kind='function'">#B0C0E0</xsl:when>
            <xsl:otherwise>#C0D0F0</xsl:otherwise>
          </xsl:choose>
        <xsl:text>"];</xsl:text>
        <xsl:call-template name="newline"/>

        <xsl:value-of select="$edgehook_qname"/>
        <xsl:text> [shape="point" label=""];</xsl:text>
        <xsl:call-template name="newline"/>

        <xsl:value-of select="$edgehook_qname"/>
        <xsl:text> -> </xsl:text>
        <xsl:value-of select="$edgelabel_qname"/>
        <xsl:text> [constraint=false penwidth=1.0 arrowhead="none" style="dotted"];</xsl:text>
        <xsl:call-template name="newline"/>

        <xsl:text>
          }
        </xsl:text>

        <xsl:text>"</xsl:text>
        <xsl:value-of select="$struct_name"/>
        <xsl:text>" -> </xsl:text>
        <xsl:value-of select="$edgelabel_qname"/>
        <xsl:text> [style="invis" weight=4];</xsl:text>
        <xsl:call-template name="newline"/>

        <xsl:text>"</xsl:text>
        <xsl:value-of select="$struct_name"/>
        <xsl:text>" -> </xsl:text>
        <xsl:value-of select="$edgehook_qname"/>
        <xsl:text> [penwidth=0.2 arrowsize=2.0 constraint=false dir="both" weight=2 arrowhead="none" arrowtail="</xsl:text>
        <xsl:choose>
          <xsl:when test="$is_aggr">diamond</xsl:when>
          <xsl:otherwise>none</xsl:otherwise>
        </xsl:choose>
        <xsl:text>"];</xsl:text>
        <xsl:call-template name="newline"/>

        <xsl:if test="$referent_name and /doxygen/compounddef/compoundname[text() = $referent_name]">
          <xsl:value-of select="$edgelabel_qname"/>
          <xsl:text> -> "</xsl:text>
          <xsl:value-of select="$referent_name"/>
          <xsl:text>" [style="invis" weight=7];</xsl:text>
          <xsl:call-template name="newline"/>

          <xsl:value-of select="$edgehook_qname"/>
          <xsl:text> -> "</xsl:text>
          <xsl:value-of select="$referent_name"/>
          <xsl:text>" [penwidth=0.2 arrowsize=2.0 constraint=false weight=2 arrowhead="vee"];</xsl:text>
          <xsl:call-template name="newline"/>
        </xsl:if>
      </xsl:if>

    </xsl:for-each>
    <xsl:text>}</xsl:text>
    <xsl:call-template name="newline"/>
  </xsl:template>

</xsl:stylesheet>
